import calendar
import datetime

import svgwrite

from github_poster.config import (
    COLOR_TUPLE,
    DEFAULT_DOM_COLOR,
    DOM_BOX_DICT,
    DOM_BOX_TUPLE,
    MONTH_NAMES,
)
from github_poster.err import BaseDrawError
from github_poster.utils import interpolate_color, make_key_times


class Drawer:

    name = "github"

    def __init__(self, p):
        self.poster = p
        self.year_size = 200 * 4.0 / 80.0
        self.year_style = f"font-size:{self.year_size}px; font-family:Arial;"
        self.year_length_style = f"font-size:{110 * 3.0 / 80.0}px; font-family:Arial;"
        self.month_names_style = "font-size:2.5px; font-family:Arial"

    @property
    def type_color_dict(self):
        """
        for multiple types
        """
        return dict(
            zip(self.poster.type_list, COLOR_TUPLE[: len(self.poster.type_list)])
        )

    def make_color(self, length_range, length):
        sp2 = self.poster.special_number.get("special_number2")
        sp1 = self.poster.special_number.get("special_number1")
        has_special = False
        if sp2 and sp1 and length:
            has_special = sp2 < length < sp1
        color_from = (
            self.poster.colors["special"]
            if has_special
            else self.poster.colors["track"]
        )
        color_to = self.poster.colors["special2"]
        diff = length_range.diameter()
        if diff == 0:
            return color_from

        return interpolate_color(
            color_from, color_to, (length - length_range.lower()) / diff
        )

    def __add_animation(self, rect, key_times, animate_index):
        values = (
            ";".join(["0"] * animate_index)
            + ";"
            + ";".join(["1"] * (len(key_times) - animate_index))
        )
        rect.add(
            svgwrite.animate.Animate(
                "opacity",
                dur=f"{self.poster.animation_time}s",
                values=values,
                keyTimes=";".join(key_times),
                repeatCount="1",
            )
        )
        return rect

    def _gen_day_box(
        self,
        dr,
        rect_x,
        rect_y,
        date_title,
        day_tracks,
        with_animation,
        key_times,
        animate_index,
    ):
        color = DEFAULT_DOM_COLOR
        if day_tracks:
            color = self.make_color(self.poster.length_range_by_date, day_tracks)
            if day_tracks >= self.poster.special_number["special_number1"]:
                color = self.poster.colors.get("special2") or self.poster.colors.get(
                    "special"
                )
            date_title = f"{date_title} {day_tracks} {self.poster.units}"
        rect = dr.rect((rect_x, rect_y), DOM_BOX_TUPLE, fill=color)
        if with_animation:
            rect = self.__add_animation(rect, key_times, animate_index)
        rect.set_desc(title=date_title)
        yield rect

    def _gen_day_boxes(
        self,
        dr,
        rect_x,
        rect_y,
        date_title,
        day_tracks,
        with_animation,
        key_times,
        animate_index,
    ):
        """
        max len(boxes) == 3 like douban see #7
        yield rect1, rect2, rect3
        or
        yield rect1, rect2
        or
        yield rect
        """
        if day_tracks:
            types_len = len(day_tracks)
            dom_tuple = DOM_BOX_DICT.get(types_len).get("dom")
            index = 0
            for _type in self.poster.type_list:
                num = day_tracks.get(_type, 0)
                length_range = self.poster.length_range_by_date_dict.get(_type, 1)
                if not num:
                    continue
                dom = dom_tuple[index]
                color = self.make_color(length_range, num)
                rect = dr.rect((rect_x, rect_y), dom, fill=color)
                date_title = f"{date_title} {num} for {_type}"
                if with_animation:
                    rect = self.__add_animation(rect, key_times, animate_index)
                rect.set_desc(title=date_title)
                yield rect
                rect_y += dom_tuple[index][1]
                index += 1
        else:
            rect = dr.rect((rect_x, rect_y), DOM_BOX_TUPLE, fill=DEFAULT_DOM_COLOR)
            if with_animation:
                rect = self.__add_animation(rect, key_times, animate_index)
            yield rect

    def _draw_one_calendar(self, dr, year, offset, _type=None):
        start_date_weekday, _ = calendar.monthrange(year, 1)
        github_rect_first_day = datetime.date(year, 1, 1)
        # Github profile the first day start from the last Monday of the last year
        # or the first Monday of this year.
        # It depands on if the first day of this year is Monday or not.
        github_rect_day = github_rect_first_day + datetime.timedelta(
            -start_date_weekday
        )
        year_length = self.poster.total_sum_year_dict.get(year, 0)
        year_length = str(int(year_length)) + f" {self.poster.units}"
        dr.add(
            dr.text(
                f"{year}" if _type is None else f"{_type}",
                insert=offset.tuple(),
                fill=self.poster.colors["text"],
                alignment_baseline="hanging",
                style=self.year_style,
            )
        )

        if not self.poster.is_multiple_type:
            dr.add(
                dr.text(
                    f"{year_length}",
                    insert=(offset.tuple()[0] + 165, offset.tuple()[1] + 5),
                    fill=self.poster.colors["text"],
                    alignment_baseline="hanging",
                    style=self.year_length_style,
                )
            )
        # add month name up to the poster one by one
        # because of svg text auto trim the spaces.
        for num, name in enumerate(MONTH_NAMES):
            dr.add(
                dr.text(
                    f"{name}",
                    insert=(offset.tuple()[0] + 15.5 * num, offset.tuple()[1] + 14),
                    fill=self.poster.colors["text"],
                    style=self.month_names_style,
                )
            )

        rect_x = 10.0
        animate_index = 1
        year_count, key_times = 0, ""
        if self.poster.with_animation:
            # set default count 10
            year_count = self.poster.year_tracks_date_count_dict.get(str(year), 10)
            key_times = make_key_times(year_count)

        # add every day of this year for 53 weeks and per week has 7 days
        for _ in range(54):
            rect_y = offset.y + self.year_size + 2
            for _ in range(7):
                if int(github_rect_day.year) > year:
                    break
                rect_y += 3.5
                date_title = str(github_rect_day)
                day_tracks = None
                if date_title in self.poster.tracks:
                    day_tracks = self.poster.tracks[date_title]

                    # tricky for may cause animate error
                    if animate_index < len(key_times) - 1:
                        animate_index += 1
                gen_box_func = (
                    self._gen_day_box
                    if len(self.poster.type_list) == 1
                    else self._gen_day_boxes
                )
                for rect in gen_box_func(
                    dr,
                    rect_x,
                    rect_y,
                    date_title,
                    day_tracks,
                    self.poster.with_animation,
                    key_times,
                    animate_index,
                ):
                    dr.add(rect)
                github_rect_day += datetime.timedelta(1)
            rect_x += 3.5
        offset.y += 3.5 * 9 + self.year_size + 1.5

    def draw(self, dr, offset, is_summary=False):
        if self.poster.tracks is None:
            raise BaseDrawError("No tracks to draw")

        if is_summary:
            for l in self.poster.loader_list:
                tracks, years = l.get_all_track_data()
                self.poster.set_tracks(tracks, years, [])
                self.poster.type_list = ["summary"]
                self.poster.special_number = {
                    "special_number1": l.special_number1,
                    "special_number2": l.special_number2,
                }
                self.poster.colors["track"] = l.track_color or "#4DD2FF"
                self.poster.units = l.unit
                self.poster.compute_track_statistics([l._type])
                self._draw_one_calendar(dr, years[0], offset, _type=l._type)
        else:
            for year in range(self.poster.years[0], self.poster.years[-1] + 1)[::-1]:
                self._draw_one_calendar(dr, year, offset)

    def draw_footer(self, dr):
        text_color = self.poster.colors["text"]
        header_style = "font-size:4px; font-family:Arial"
        x = 10
        y = self.poster.height - 2.5
        index = 0
        for _type in self.poster.type_list:
            dr.add(dr.rect((x, y - 2.5), DOM_BOX_TUPLE, fill=COLOR_TUPLE[index][0]))
            dr.add(
                dr.text(
                    f": {_type}",
                    insert=(x + 3, y),
                    fill=text_color,
                    style=header_style,
                )
            )
            x += 20
            index += 1



import time

import requests
from pendulum import parse, period

from github_poster.html_parser import GitLabParser
from github_poster.loader.base_loader import BaseLoader, LoadError
from github_poster.loader.config import GITLAB_LATEST_URL, GITLAB_ONE_DAY_URL


class GitLabLoader(BaseLoader):
    track_color = "#ACD5F2"
    unit = "cons"

    def __init__(self, from_year, to_year, _type, **kwargs):
        super().__init__(from_year, to_year, _type)
        self.user_name = kwargs.get("gitlab_user_name", "")
        self.gitlab_base_url = kwargs.get("base_url") or "https://gitlab.com"
        self.gitlab_session = kwargs.get("session")
        self.left_dates = []

    @classmethod
    def add_loader_arguments(cls, parser, optional):
        parser.add_argument(
            "--gitlab_user_name",
            dest="gitlab_user_name",
            type=str,
            required=optional,
            help="",
        )

        parser.add_argument(
            "--base_url",
            dest="base_url",
            type=str,
            default="https://gitlab.com",
            help="specify the base url of your self-managed gitlab",
        )
        parser.add_argument(
            "--session",
            dest="session",
            type=str,
            default="",
            help="use gitlab_session from Cookies "
            "if your gitlab instance needs to sign in",
        )

    def _make_left_dates(self, last_date):
        dates = list(period(parse(f"{self.from_year}-01-01"), parse(last_date)))
        self.left_dates = [i.to_date_string() for i in dates]

    def _set_cookies(self):
        if self.gitlab_session:
            return {"_gitlab_session": self.gitlab_session}

        return {}

    def make_latest_date_dict(self):
        try:
            r = requests.get(
                GITLAB_LATEST_URL.format(
                    gitlab_base_url=self.gitlab_base_url, user_name=self.user_name
                ),
                cookies=self._set_cookies(),
            )
            date_dict = r.json()
            min_date = min(date_dict.keys())
            self.number_by_date_dict = date_dict
            if self.from_year > int(min_date[:4]):
                return
            self._make_left_dates(min_date)
        except Exception as e:
            raise LoadError(f"Can not get gitlab data error: {str(e)}")

    def make_left_data_dict(self):
        p = GitLabParser()
        for d in self.left_dates:
            try:
                r = requests.get(
                    GITLAB_ONE_DAY_URL.format(
                        gitlab_base_url=self.gitlab_base_url,
                        user_name=self.user_name,
                        date_str=d,
                    ),
                    cookies=self._set_cookies(),
                )
                # spider rule
                time.sleep(0.1)
                p.feed(r.text)
                self.number_by_date_dict[d] = len(p.lis)
            except Exception:
                # what fucking things happened just pass
                pass

    def make_track_dict(self):
        self.make_latest_date_dict()
        self.make_left_data_dict()
        for _, v in self.number_by_date_dict.items():
            self.number_list.append(v)

    def get_all_track_data(self):
        self.make_track_dict()
        self.make_special_number()
        return self.number_by_date_dict, self.year_list



import datetime
import os
from collections import defaultdict

import gpxpy

from github_poster.loader.base_loader import BaseLoader, LoadError
from github_poster.loader.config import GPX_ACTIVITY_NAME_TUPLE


class GPXLoader(BaseLoader):
    unit = "km"

    def __init__(self, from_year, to_year, _type, **kwargs):
        super().__init__(from_year, to_year, _type)
        self.number_by_date_dict = defaultdict(float)
        self.before = None
        self.after = None
        self.base_dir = kwargs.get("gpx_dir", "")

    @classmethod
    def add_loader_arguments(cls, parser, optional):
        parser.add_argument(
            "--gpx_dir",
            dest="gpx_dir",
            metavar="DIR",
            type=str,
            default="GPX_FOLDER",
            help="Directory containing GPX files",
        )

    def _make_year_before_after(self):
        self.before = datetime.datetime(int(self.to_year) + 1, 1, 1)
        self.after = datetime.datetime(int(self.from_year), 1, 1)

    def _list_gpx_files(self):
        base_dir = os.path.abspath(self.base_dir)
        if not os.path.isdir(base_dir):
            raise LoadError(f"Not a directory: {base_dir}")
        for name in os.listdir(base_dir):
            if name.startswith("."):
                continue
            path_name = os.path.join(base_dir, name)
            if name.endswith(".gpx") and os.path.isfile(path_name):
                yield path_name

    def __parse_gpx(self, file_name):
        with open(file_name) as f:
            gpx = gpxpy.parse(f)
            try:
                start_time, _ = gpx.get_time_bounds()
                # timezone offset
                start_time = self.adjust_time(start_time)
                distance = gpx.length_2d()
            except Exception as e:
                print(f"Something is wrong when loading file {file_name}", str(e))
        return start_time.date(), distance

    def get_api_data(self):
        self._make_year_before_after()
        files = list(self._list_gpx_files())
        print("Loading your gpx files it may take a little time please wait")
        for f in files:
            date, distance = self.__parse_gpx(f)
            # filter
            if date.year < self.from_year or date.year > self.to_year:
                continue
            yield GPX_ACTIVITY_NAME_TUPLE(date, distance)

    def make_track_dict(self):
        tracks = list(self.get_api_data())
        for t in tracks:
            num = round(float(t.distance) / 1000, 2)
            self.number_by_date_dict[str(t.date)] += num
            self.number_list.append(num)
        return tracks

    def get_all_track_data(self):
        self.make_track_dict()
        self.make_special_number()
        return self.number_by_date_dict, self.year_list



import hashlib
import json

import requests

from github_poster.loader.base_loader import BaseLoader, LoadError
from github_poster.loader.config import (
    CICHANG_CLAENDAR_URL,
    CICHANG_COVERT_URL,
    CICHANG_LOGIN_URL,
    HJ_APPKEY,
)


class CiChangLoader(BaseLoader):
    unit = "words"

    def __init__(self, from_year, to_year, _type, **kwargs):
        super().__init__(from_year, to_year, _type)
        self.user_name = kwargs.get("cichang_user_name", "")
        self.password = kwargs.get("cichang_password", "")
        self.s = requests.Session()

    @classmethod
    def add_loader_arguments(cls, parser, optional):
        parser.add_argument(
            "--cichang_user_name",
            dest="cichang_user_name",
            type=str,
            required=optional,
            help="The username of CiChang",
        )
        parser.add_argument(
            "--cichang_password",
            dest="cichang_password",
            type=str,
            required=optional,
            help="The password of CiChang",
        )

    @staticmethod
    def _md5_encode(string):
        m = hashlib.md5()
        m.update(string.encode())
        return m.hexdigest()

    def login(self):
        password_md5 = self._md5_encode(self.password)
        r = self.s.get(
            CICHANG_LOGIN_URL.format(user_name=self.user_name, password=password_md5)
        )
        if not r.ok:
            raise LoadError(f"Someting is wrong to login -- {r.text}")
        club_auth_cookie = r.json()["Data"]["Cookie"]
        data = {"club_auth_cookie": club_auth_cookie}
        headers = {"hj_appkey": HJ_APPKEY, "Content-Type": "application/json"}
        # real login to get real token
        r = self.s.post(CICHANG_COVERT_URL, headers=headers, data=json.dumps(data))
        if not r.ok:
            raise LoadError(f"Get real token failed -- {r.text}")
        data = r.json()["data"]
        access_token = data["access_token"]
        user_id = data["user_id"]
        headers["Access-Token"] = access_token
        self.s.headers = headers
        self.user_id = user_id

    def get_api_data(self):
        month_list = self.make_month_list()
        data_list = []
        for m in month_list:
            r = self.s.get(
                CICHANG_CLAENDAR_URL.format(
                    user_id=self.user_id,
                    start_date=m.to_date_string(),
                    end_date=m.end_of("month").to_date_string(),
                )
            )
            if not r.ok:
                print(f"get cichang calendar api failed {str(r.text)}")
            try:
                data_list.extend(r.json()["data"]["studyCountDays"])
            except Exception:
                # just pass for now
                pass
        return data_list

    def make_track_dict(self):
        data_list = self.get_api_data()
        for d in data_list:
            if d:
                number = d["studyCount"]
                if number:
                    self.number_by_date_dict[d["studyDate"].replace("/", "-")] = number
                    self.number_list.append(number)

    def get_all_track_data(self):
        self.login()
        self.make_track_dict()
        self.make_special_number()
        return self.number_by_date_dict, self.year_list



import datetime
from collections import defaultdict

import stravalib

from github_poster.loader.base_loader import BaseLoader, LoadError


class StravaLoader(BaseLoader):
    unit = "km"

    def __init__(self, from_year, to_year, _type, **kwargs):
        super().__init__(from_year, to_year, _type)
        self.before = None
        self.after = None
        self.number_by_date_dict = defaultdict(float)
        self.client = stravalib.Client()
        self.client_id = kwargs.get("strava_client_id", "")
        self.client_secret = kwargs.get("strava_client_secret", "")
        self.refresh_token = kwargs.get("strava_refresh_token", "")
        self.strava_access = False

    @classmethod
    def add_loader_arguments(cls, parser, optional):
        parser.add_argument(
            "--strava_client_id",
            dest="strava_client_id",
            type=str,
            required=optional,
            help="",
        )
        parser.add_argument(
            "--strava_client_secret",
            dest="strava_client_secret",
            type=str,
            required=optional,
            help="",
        )
        parser.add_argument(
            "--strava_refresh_token",
            dest="strava_refresh_token",
            type=str,
            required=optional,
            help="",
        )

    def _make_year_before_after(self):
        self.before = datetime.datetime(int(self.to_year) + 1, 1, 1)
        self.after = datetime.datetime(int(self.from_year), 1, 1)

    def _get_access(self):
        try:
            response = self.client.refresh_access_token(
                client_id=self.client_id,
                client_secret=self.client_secret,
                refresh_token=self.refresh_token,
            )
        except Exception:
            raise LoadError("Something is wrong with your auth please check")

        self.access_token = response["access_token"]
        self.refresh_token = response["refresh_token"]
        self.client.access_token = response["access_token"]
        self.strava_access = True

    def get_api_data(self):
        if not self.strava_access:
            self._get_access()
        # make year range
        self._make_year_before_after()
        return self.client.get_activities(before=self.before, after=self.after)

    def make_track_dict(self):
        tracks = list(self.get_api_data())
        for t in tracks:
            num = round(float(t.distance) / 1000, 2)
            self.number_by_date_dict[str(t.start_date_local.date())] += num
            self.number_list.append(num)
        return tracks

    def get_all_track_data(self):
        self.make_track_dict()
        self.make_special_number()
        return self.number_by_date_dict, self.year_list



import pendulum
from github import Github

from github_poster.loader.base_loader import BaseLoader


class GitHubIssuesLoader(BaseLoader):
    def __init__(self, from_year, to_year, _type, **kwargs):
        super().__init__(from_year, to_year, _type)
        self.issue_number = int(kwargs.get("issue_number", "1"))
        self.repo_name = kwargs.get("repo_name", "")
        # for private repo
        self.token = kwargs.get("github_token", "")

    @classmethod
    def add_loader_arguments(cls, parser, optional):
        parser.add_argument(
            "--issue_number",
            dest="issue_number",
            type=str,
            required=optional,
            help="The issue number",
        )
        parser.add_argument(
            "--repo_name",
            dest="repo_name",
            type=str,
            required=optional,
            help="The repo name",
        )
        parser.add_argument(
            "--github_token",
            dest="github_token",
            type=str,
            default="",
            help="The GitHub token, required by private repo",
        )

    @staticmethod
    def __map_func(comment):
        data = comment.body.splitlines()[0]
        try:
            return int(data)
        except (ValueError, TypeError):
            return 0

    def get_api_data(self):
        if self.token:
            u = Github(self.token)
            me = u.get_user().login
        else:
            u = Github()
            me = "someone"
        repo = u.get_repo(self.repo_name)
        comments = repo.get_issue(self.issue_number).get_comments()
        for c in comments:
            if (
                c.user.login == me
                and pendulum.instance(c.created_at).in_timezone(self.time_zone).year
                in self.year_list
            ):
                yield c

    def make_track_dict(self):
        data_list = self.get_api_data()
        for d in data_list:
            date_str = (
                pendulum.instance(d.created_at)
                .in_timezone(self.time_zone)
                .to_date_string()
            )
            number = self.__map_func(d)
            if number:
                self.number_by_date_dict[date_str] += number
                self.number_list.append(number)

    def get_all_track_data(self):
        self.make_track_dict()
        self.make_special_number()
        return self.number_by_date_dict, self.year_list



import json
from collections import defaultdict

from github_poster.loader.base_loader import BaseLoader


class JsonLoader(BaseLoader):
    unit = "times"

    def __init__(self, from_year, to_year, _type, **kwargs):
        super().__init__(from_year, to_year, _type)
        self.number_by_date_dict = defaultdict(int)
        self.json_file = kwargs.get("json_file")

    @classmethod
    def add_loader_arguments(cls, parser, optional):
        parser.add_argument(
            "--json_file",
            dest="json_file",
            type=str,
            default="data.json",
            help="json file path",
        )

    def get_api_data(self):
        pass

    def make_track_dict(self):
        with open(self.json_file, "r") as f:
            tracks = json.load(f)
        self.number_by_date_dict = tracks
        for _, v in self.number_by_date_dict.items():
            self.number_list.append(v)
        return tracks

    def get_all_track_data(self):
        self.make_track_dict()
        self.make_special_number()
        return self.number_by_date_dict, self.year_list



"""Create a poster from track data."""
from collections import defaultdict

import svgwrite

from github_poster.structures import XY, ValueRange


class Poster:
    def __init__(self):
        self.title = None
        self.tracks = {}
        self.type_list = []
        self.loader_list = []
        self.length_range_by_date = ValueRange()
        self.length_range_by_date_dict = defaultdict(ValueRange)
        self.units = "metric"
        self.colors = {
            "background": "#222222",
            "text": "#FFFFFF",
            "special": "#FFFF00",
            "track": "#4DD2FF",
        }
        self.width = 200
        self.height = 300
        self.years = None
        # maybe support more type
        self.tracks_drawer = None
        self.trans = None
        self.with_animation = False
        self.animation_time = 10
        self.year_tracks_date_count_dict = defaultdict(int)
        self.year_tracks_type_dict = defaultdict(dict)

        # for year sumary
        self.is_summary = False

    def set_tracks(self, tracks, years, type_list):
        self.type_list.extend(type_list)
        self.tracks = tracks
        self.years = years
        # for mutiple types...
        # TODO maybe refactor another class later
        for date, num in tracks.items():
            self.year_tracks_date_count_dict[date[:4]] += 1
            if type(num) is dict:
                for k, v in num.items():
                    self.length_range_by_date_dict[k].extend(v)
            else:
                self.length_range_by_date.extend(num)
        for t in type_list:
            self.compute_track_statistics(t)

    @property
    def is_multiple_type(self):
        return len(self.type_list) > 1

    def set_with_animation(self, with_animation):
        self.with_animation = with_animation

    def set_animation_time(self, animation_time):
        self.animation_time = animation_time

    def draw(self, drawer, output):
        assert self.type_list, "type_list is empty"
        if drawer.name == "circular":
            self._draw_circular(drawer, output)
        else:
            self._draw_github(drawer, output)

    def _draw_circular(self, drawer, output):
        self.tracks_drawer = drawer
        d = svgwrite.Drawing(output, (f"{self.width}mm", f"{self.height}mm"))
        d.viewbox(0, 0, self.width, self.height)
        d.add(d.rect((0, 0), (self.width, self.height), fill=self.colors["background"]))
        self.__draw_tracks(d, XY(100, 100))
        d.save()

    def _draw_github(self, drawer, output):
        height = self.height
        width = self.width
        self.tracks_drawer = drawer
        d = svgwrite.Drawing(output, (f"{width}mm", f"{height}mm"))
        d.viewbox(0, 0, self.width, height)
        d.add(d.rect((0, 0), (width, height), fill=self.colors["background"]))
        self.__draw_header(d)
        self.__draw_tracks(d, XY(10, 30))
        # for multiple types show
        if self.is_multiple_type:
            self.__draw_footer(d)
        d.save()

    def __draw_tracks(self, d, offset):
        self.tracks_drawer.draw(d, offset, self.is_summary)

    def __draw_header(self, d):
        text_color = self.colors["text"]
        title_style = "font-size:12px; font-family:Arial; font-weight:bold;"
        d.add(d.text(self.title, insert=(10, 20), fill=text_color, style=title_style))

    def __draw_footer(self, d):
        self.tracks_drawer.draw_footer(d)

    def compute_track_statistics(self, t):
        total_sum_year_dict = defaultdict(int)
        for date, num in self.tracks.items():
            if type(num) is dict:
                total_sum_year_dict[int(date[:4])] += num.get(t, 0)
            else:
                total_sum_year_dict[int(date[:4])] += num
        self.total_sum_year_dict = total_sum_year_dict
        return total_sum_year_dict



import pendulum
import requests

from github_poster.loader.base_loader import BaseLoader
from github_poster.loader.config import DUOLINGO_CALENDAR_API, DUOLINGO_LOGIN_URL


class DuolingoLoader(BaseLoader):
    unit = "XP"
    track_color = "#78C800"

    def __init__(self, from_year, to_year, _type, **kwargs):
        super().__init__(from_year, to_year, _type)
        self.user_name = kwargs.get("duolingo_user_name", "")
        self.password = kwargs.get("duolingo_password", "")
        self.session = requests.Session()
        # duolingo name to get the calendar
        self.duolingo_id = ""

    @classmethod
    def add_loader_arguments(cls, parser, optional):
        parser.add_argument(
            "--duolingo_user_name",
            dest="duolingo_user_name",
            type=str,
            help="",
            required=optional,
        )
        parser.add_argument(
            "--duolingo_password",
            dest="duolingo_password",
            type=str,
            help="",
            required=optional,
        )

    def login(self):
        r = self.session.post(
            DUOLINGO_LOGIN_URL,
            params={"login": self.user_name, "password": self.password},
        )
        if r.status_code != 200:
            raise Exception("Login failed")
        self.duolingo_id = r.json()["user_id"]

    def get_api_data(self):
        month_list = self.make_month_list()
        for m in month_list:
            r = self.session.get(
                DUOLINGO_CALENDAR_API.format(
                    user_id=self.duolingo_id,
                    start_date=m.to_date_string(),
                    end_date=m.end_of("month").to_date_string(),
                )
            )
            if not r.ok:
                print(f"get duolingo calendar api failed {str(r.text)}")
            try:
                yield from r.json()["summaries"]
            except Exception:
                # just pass for now
                pass

    def make_track_dict(self):
        data_list = self.get_api_data()
        for d in data_list:
            date_str = pendulum.from_timestamp(d["date"]).to_date_string()
            number = d["gainedXp"]
            if number:
                self.number_by_date_dict[date_str] = number
                self.number_list.append(number)

    def get_all_track_data(self):
        self.login()
        self.make_track_dict()
        self.make_special_number()
        return self.number_by_date_dict, self.year_list



import json
import os
from collections import defaultdict

import pendulum

from github_poster.loader.base_loader import BaseLoader


class YouTubeLoader(BaseLoader):
    track_color = "#FFFFFF"
    unit = "videos"

    def __init__(self, from_year, to_year, _type, **kwargs):
        super().__init__(from_year, to_year, _type)
        self.number_by_date_dict = defaultdict(int)
        self.youtube_file = kwargs.get("youtube_history_file")

    @classmethod
    def add_loader_arguments(cls, parser, optional):
        parser.add_argument(
            "--youtube_history_file",
            dest="youtube_history_file",
            type=str,
            default=os.path.join("IN_FOLDER", "watch-history.json"),
            help="youtube history file path",
        )

    def _parse_youtube_history(self):
        base_file = self.youtube_file
        data_list = []
        with open(base_file) as f:
            data_list = json.load(f)
        return data_list

    def get_api_data(self):
        return self._parse_youtube_history()

    def make_track_dict(self):
        tracks = self.get_api_data()
        for t in tracks:
            date_time = pendulum.parse(t["time"], tz=self.time_zone)
            if date_time.year < self.from_year:
                break
            date = date_time.to_date_string()
            self.number_by_date_dict[date] += 1
        for _, v in self.number_by_date_dict.items():
            self.number_list.append(v)
        return tracks

    def get_all_track_data(self):
        self.make_track_dict()
        self.make_special_number()
        return self.number_by_date_dict, self.year_list



from github_poster.loader.bilibili_loader import BilibiliLoader
from github_poster.loader.cichang_loader import CiChangLoader
from github_poster.loader.dota2_loader import Dota2Loader
from github_poster.loader.duolingo_loader import DuolingoLoader
from github_poster.loader.forest_loader import ForestLoader
from github_poster.loader.from_github_issue_loader import GitHubIssuesLoader
from github_poster.loader.garmin_loader import GarminLoader
from github_poster.loader.github_loader import GitHubLoader
from github_poster.loader.gitlab_loader import GitLabLoader
from github_poster.loader.gpx_loader import GPXLoader
from github_poster.loader.jike_loader import JikeLoader
from github_poster.loader.json_loader import JsonLoader
from github_poster.loader.kindle_loader import KindleLoader
from github_poster.loader.leetcode_loader import LeetcodeLoader
from github_poster.loader.multiple_loader import MutipleLoader
from github_poster.loader.nrc_loader import NRCLoader
from github_poster.loader.ns_loader import NSLoader
from github_poster.loader.shanbay_loader import ShanBayLoader
from github_poster.loader.strava_loader import StravaLoader
from github_poster.loader.summary_loader import SummaryLoader
from github_poster.loader.twitter_loader import TwitterLoader
from github_poster.loader.wakatime_loader import WakaTimeLoader
from github_poster.loader.youtube_loader import YouTubeLoader

LOADER_DICT = {
    "duolingo": DuolingoLoader,
    "shanbay": ShanBayLoader,
    "strava": StravaLoader,
    "cichang": CiChangLoader,
    "ns": NSLoader,
    "gpx": GPXLoader,
    "issue": GitHubIssuesLoader,
    "leetcode": LeetcodeLoader,
    "twitter": TwitterLoader,
    "youtube": YouTubeLoader,
    "bilibili": BilibiliLoader,
    "github": GitHubLoader,
    "gitlab": GitLabLoader,
    "kindle": KindleLoader,
    "wakatime": WakaTimeLoader,
    "dota2": Dota2Loader,
    "multiple": MutipleLoader,
    "nike": NRCLoader,
    "garmin": GarminLoader,
    "forest": ForestLoader,
    "json": JsonLoader,
    "jike": JikeLoader,
    "summary": SummaryLoader,
}

__all__ = (
    "BilibiliLoader",
    "CiChangLoader",
    "Dota2Loader",
    "DuolingoLoader",
    "GitHubIssuesLoader",
    "GitHubLoader",
    "GitLabLoader",
    "GPXLoader",
    "KindleLoader",
    "LeetcodeLoader",
    "NSLoader",
    "ShanBayLoader",
    "StravaLoader",
    "TwitterLoader",
    "WakaTimeLoader",
    "YouTubeLoader",
    "MutipleLoader",
    "NRCLoader",
    "LOADER_DICT",
    "ForestLoader",
    "GarminLoader",
    "JsonLoader",
    "JikeLoader",
    "SummaryLoader",
)



import re
from collections import namedtuple

# China timezone if you are from others please change this
TIME_ZONE = "Asia/Shanghai"

# shanbay -- no need to login
SHANBAY_CALENDAR_API = (
    "https://apiv3.shanbay.com/uc/checkin/calendar/dates/"
    "?user_id={user_name}&start_date={start_date}&end_date={end_date}"
)

# duolingo -- no need to login
DUOLINGO_CALENDAR_API = (
    "https://ios-api-2.duolingo.com/2017-06-30/users/{user_id}/xp_summaries"
    "?endDate={end_date}&startDate={start_date}&timezone=Asia/Shanghai"
)
DUOLINGO_LOGIN_URL = "https://www.duolingo.com/login"

# cichang -- need to login
HJ_APPKEY = "45fd17e02003d89bee7f046bb494de13"
CICHANG_LOGIN_URL = (
    "https://pass.hujiang.com/Handler/UCenter.json?action=Login&isapp=true"
    "&language=zh_CN&password={password}&timezone=8&user_domain=hj&username={user_name}"
)
CICHANG_COVERT_URL = "https://pass-cdn.hjapi.com/v1.1/access_token/convert"
CICHANG_CLAENDAR_URL = (
    "https://cichang.hjapi.com/v3/user/center/"
    "?userId={user_id}&startDate={start_date}&endDate={end_date}"
)

# switch -- need to packet sniffer (to get device id and token)
TOKEN_API_URL = "https://accounts.nintendo.com/connect/1.0.0/api/token"
NS_CLIENT_ID = "54789befb391a838"
NS_GRANT_TYPE = "urn:ietf:params:oauth:grant-type:jwt-bearer-session-token"
NS_TOKEN_URL = "https://accounts.nintendo.com/connect/1.0.0/api/token"
NS_CLAENDAR_URL = (
    "https://api-lp1.pctl.srv.nintendo.net/moon/v1/devices"
    "/{device_id}/monthly_summaries/{month}"
)

# gpx need a gpx folder
GPX_ACTIVITY_NAME_TUPLE = namedtuple("activity", "date distance")

# GitHub for furture use
GITHUB_BASE_URL = "https://github.com"

# LeetCode
LEETCODE_SUBMISSIONS_URL = (
    "https://leetcode.com/api/submissions/?offset={offset}&limit=20&lastkey={last_key}"
)
LEETCODE_CN_SUBMISSIONS_URL = (
    "https://leetcode-cn.com/api/submissions/"
    "?offset={offset}&limit=20&lastkey={last_key}"
)

# Bilibili
BILIBILI_HISTORY_URL = (
    "https://api.bilibili.com/x/web-interface/history/cursor"
    "?max={max_oid}&view_at={view_at}&business=archive"
)

# GitHub
GITHUB_CONTRIBUCTIONS_URL = (
    "https://github.com/users/{user_name}/contributions?from={start_day}&to={end_day}"
)

# GitLab
GITLAB_LATEST_URL = "{gitlab_base_url}/users/{user_name}/calendar.json"
GITLAB_ONE_DAY_URL = (
    "{gitlab_base_url}/users/{user_name}/calendar_activities?date={date_str}"
)

# Kindle
KINDLE_HISTORY_URL = "https://www.amazon.com/kindle/reading/insights"
KINDLE_CN_HISTORY_URL = "https://www.amazon.cn/kindle/reading/insights"
KINDLE_HEADER = {
    "User-Agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 14_6 like Mac OS X) "
    "AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/1AE148",
}

# WakaTime
WAKATIME_SUMMARY_URL = (
    "https://wakatime.com/api/v1/users/current/summaries"
    "?api_key={wakatime_key}&start={from_year}&end={to_year}"
)

# Dota2
DOTA2_CALENDAR_API = "https://api.opendota.com/api/players/{dota2_id}/matches"

# NRC (nike)
NIKE_TOKEN_REFRESH_URL = "https://unite.nike.com/tokenRefresh"
NIKE_CLIENT_ID = "HlHa2Cje3ctlaOqnxvgZXNaAs7T9nAuH"
NIKE_BASE_URL = "https://api.nike.com/sport/v3/me"

# forest(need to login)
FOREST_URL_HEAD = "https://forest.dc.upwardsware.com"
FOREST_LOGIN_URL = FOREST_URL_HEAD + "/api/v1/sessions"
FOREST_CLAENDAR_URL = (
    FOREST_URL_HEAD
    + "/api/v1/plants/updated_plants?update_since={date}&seekruid={user_id}"
)

# jike
JIKE_GRAPHQL_URL = "https://web-api.okjike.com/api/graphql"
JIKE_PERSON_URL = "https://web.okjike.com/u/{user_id}"

# argument dict
ARGUMENT_DICT = {
    "strava": [],
    "shanbay": [],
    "duolingo": [
        {
            "argument": "--duolingo_user_name",
            "dest": "duolingo_user_name",
            "type": str,
            "help": "duolingo user name",
            "required": True,
        },
        {
            "argument": "--duolingo_password",
            "dest": "duolingo_password",
            "type": str,
            "help": "duolingo password",
            "required": True,
        },
    ],
    "cichang": [],
    "switch": [],
    "gpx": [],
    "github": [
        {
            "argument": "--github_user_name",
            "dest": "github_user_name",
            "type": str,
            "required": True,
            "help": "GitHub user name",
        }
    ],
    "leetcode": [],
    "bilibili": [
        {
            "argument": "--bilibili_cookie",
            "dest": "bilibili_cookie",
            "type": str,
            "required": True,
            "help": "The cookie for the bilibili website(XHR)",
        }
    ],
    "kindle": [],
    "wakatime": [],
    "dota2": [],
    "nike": [],
    "forest": [],
    "jike": [],
}



import html.parser


class GitHubParser(html.parser.HTMLParser):
    def __init__(self):
        html.parser.HTMLParser.__init__(self)
        self.rects = []

    def handle_starttag(self, tag, attrs):
        if tag == "rect":
            self.rects.append(dict(attrs))

    def make_contribution_dict(self, text):
        self.feed(text)
        number_by_date_dict = {}
        for r in self.rects:
            if r.get("data-count"):
                try:
                    number = int(r.get("data-count"))
                    if number:
                        number_by_date_dict[r["data-date"]] = number
                except Exception:
                    # just ignore it
                    pass
        return number_by_date_dict



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# some code from https://github.com/flopp/GpxTrackPoster

import argparse
import os

from github_poster.circluar_drawer import CircularDrawer
from github_poster.config import TYPE_INFO_DICT
from github_poster.drawer import Drawer
from github_poster.loader import LOADER_DICT
from github_poster.poster import Poster
from github_poster.skyline import Skyline
from github_poster.utils import parse_years

OUT_FOLDER = os.path.join(os.getcwd(), "OUT_FOLDER")


def main():
    """Handle command line arguments and call other modules as needed."""
    p = Poster()
    args_parser = argparse.ArgumentParser()
    subparser = args_parser.add_subparsers()

    # temp list to remove the summary type
    temp_list = list(LOADER_DICT.values()).copy()
    temp_list.remove(LOADER_DICT.get("summary"))
    temp_list.remove(LOADER_DICT.get("multiple"))

    for type_, l in LOADER_DICT.items():  # l -> loader class
        l._type = type_
        parser = subparser.add_parser(name=type_)
        parser.set_defaults(type=type_, loader=l)
        optional = True
        if type_ in ["summary", "multiple"]:
            l.parser_loader_list = list(temp_list)
            optional = False
        l.add_arguments(parser, optional)

    args = args_parser.parse_args()
    # without title
    no_title_types = ("issue", "multiple", "json")

    p.colors = {
        "background": args.background_color,
        "track": args.loader.track_color  # some type has default color
        or args.track_color,
        "special": args.special_color1,
        "special2": args.special_color2 or args.special_color,
        "text": args.text_color,
    }
    # set animate
    p.set_with_animation(args.with_animation)
    p.set_animation_time(args.animation_time)
    p.units = args.loader.unit
    from_year, to_year = parse_years(args.year)
    args_dict = dict(args._get_kwargs())
    loader = LOADER_DICT.get(args.type, "duolingo")(
        from_year, to_year, args.type, **args_dict
    )
    type_list = [args.type]
    # for multiple types or year summary
    if args.type in ["multiple", "summary"]:
        types_list = args_dict.get("types").split(",")
        # trim drop the spaces
        type_list = [t.replace(" ", "") for t in types_list]
        if args.with_skyline or args.is_circular:
            raise Exception("Skyline or Circular does not support for multiple types")
        if args.type == "multiple":
            assert len(types_list) <= 3
        for t in type_list:
            if t not in LOADER_DICT:
                raise Exception(f"{t} must in support loader types")
            loader.set_loader_list(
                LOADER_DICT.get(t)(from_year, to_year, t, **args_dict)
            )
            p.loader_list = loader.loader_list

    if args.type != "summary":
        tracks, years = loader.get_all_track_data()
        p.set_tracks(tracks, years, type_list)
    else:
        p.set_tracks({}, [to_year], type_list)

    # set title
    # we don't know issue content so use name
    p.title = (
        f"{args.me } {str(to_year) + ' ' if args.type=='summary' else ''}"
        + TYPE_INFO_DICT.get(args.type, args.type)
        if args.type not in no_title_types and args.without_type_name
        else args.me
    )

    p.special_number = {
        "special_number1": loader.special_number1,
        "special_number2": loader.special_number2,
    }
    if args.special_number1:
        p.special_number["special_number1"] = args.special_number1
    if args.special_number2:
        p.special_number["special_number2"] = args.special_number2
    # the length of this poster
    poster_length = len(p.years) if args.type != "summary" else len(loader.loader_list)
    p.height = 35 + poster_length * 43
    if not os.path.exists(OUT_FOLDER):
        os.mkdir(OUT_FOLDER)
    # support different issues, maybe better way
    file_name = str(args.type)

    # make different drawer
    is_circular = args.is_circular
    d = CircularDrawer if is_circular else Drawer
    if args.type == "issue":
        issue_number = args_dict.get("issue_number", "1")
        repo_name = args_dict.get("repo_name", "").replace("/", "_")
        file_name = f"issue_{repo_name}_{issue_number}"
    if args.type == "summary":
        file_name = f"summary_{to_year}"
        p.is_summary = True
    if is_circular:
        file_name = f"{file_name}_circular"

        # circular type is 120*120 square
        p.height = 120
        p.width = 120

    file_name = f"{file_name}.svg"

    # for summary we have different logic #TODO refactor
    p.draw(d(p), os.path.join(OUT_FOLDER, file_name))

    # generate skyline
    if args.with_skyline:
        if args.skyline_year:
            year = args.skyline_year
        else:
            year = years[-1]
        # filter data
        number_by_date_dict = {k: v for k, v in tracks.items() if k[:4] == str(year)}
        skyline_name = ""
        if args.skyline_with_name:
            skyline_name = args.me
        s = Skyline(
            os.path.join(OUT_FOLDER, f"{year}_{str(args.type)}" + ".stl"),
            year,
            args.type,
            number_by_date_dict,
            skyline_name,
        )
        s.type_info_dict = TYPE_INFO_DICT
        s.make_skyline()


if __name__ == "__main__":
    main()



import requests

from github_poster.loader.base_loader import BaseLoader
from github_poster.loader.config import SHANBAY_CALENDAR_API


class ShanBayLoader(BaseLoader):
    track_color = "#33C6A4"
    unit = "days"

    def __init__(self, from_year, to_year, _type, **kwargs):
        super().__init__(from_year, to_year, _type)
        self.user_name = kwargs.get("shanbay_user_name", "")

    @classmethod
    def add_loader_arguments(cls, parser, optional):
        parser.add_argument(
            "--shanbay_user_name",
            dest="shanbay_user_name",
            type=str,
            required=optional,
            help="",
        )

    def get_api_data(self):
        month_list = self.make_month_list()
        for m in month_list:
            r = requests.get(
                SHANBAY_CALENDAR_API.format(
                    user_name=self.user_name,
                    start_date=m.to_date_string(),
                    end_date=m.end_of("month").to_date_string(),
                )
            )
            if not r.ok:
                print(f"get shanbay calendar api failed {str(r.text)}")
            try:
                yield from r.json()["logs"]
            except Exception:
                # just pass for now
                pass

    def make_track_dict(self):
        data_list = self.get_api_data()
        for d in data_list:
            if d:
                self.number_by_date_dict[d["date"]] = 1
                self.number_list.append(1)

    def get_all_track_data(self):
        self.make_track_dict()
        self.make_special_number()
        return self.number_by_date_dict, self.year_list



import time
from collections import defaultdict
from http.cookies import SimpleCookie

import pendulum
import requests

from github_poster.loader.base_loader import BaseLoader, LoadError
from github_poster.loader.config import BILIBILI_HISTORY_URL


class BilibiliLoader(BaseLoader):
    track_color = "#FB7299"
    unit = "videos"

    def __init__(self, from_year, to_year, _type, **kwargs):
        super().__init__(from_year, to_year, _type)
        self.number_by_date_dict = defaultdict(int)
        self.session = requests.Session()
        self.bilibili_cookie = kwargs.get("bilibili_cookie", "")

    @classmethod
    def add_loader_arguments(cls, parser, optional):
        parser.add_argument(
            "--bilibili_cookie",
            dest="bilibili_cookie",
            type=str,
            required=optional,
            help="The cookie for the bilibili website(XHR)",
        )

    def _parse_bilibili_cookie(self):
        cookie = SimpleCookie()
        cookie.load(self.bilibili_cookie)
        cookies_dict = {}
        cookiejar = None
        for key, morsel in cookie.items():
            cookies_dict[key] = morsel.value
            cookiejar = requests.utils.cookiejar_from_dict(
                cookies_dict, cookiejar=None, overwrite=True
            )
        return cookiejar

    def get_api_data(self, max_oid="", view_at="", data_list=[]):
        r = self.session.get(
            BILIBILI_HISTORY_URL.format(max_oid=max_oid, view_at=view_at)
        )
        if not r.ok:
            raise LoadError(
                "Can not get bilibili history data, please check your cookie"
            )
        data = r.json()["data"]
        if not data["list"]:
            return data_list
        lst = data["list"]
        max_oid = lst[-1]["history"]["oid"]
        view_at = lst[-1]["view_at"]
        data_list.extend(lst)
        # spider rule
        time.sleep(0.1)
        return self.get_api_data(max_oid=max_oid, view_at=view_at, data_list=data_list)

    def make_track_dict(self):
        data_list = self.get_api_data()
        for d in data_list:
            date_str = pendulum.from_timestamp(
                d["view_at"], tz=self.time_zone
            ).to_date_string()
            self.number_by_date_dict[date_str] += 1
        for _, v in self.number_by_date_dict.items():
            self.number_list.append(v)

    def get_all_track_data(self):
        # first we need to activate the session with cookie str from `chrome`
        self.session.cookies = self._parse_bilibili_cookie()

        self.make_track_dict()
        self.make_special_number()
        return self.number_by_date_dict, self.year_list



from datetime import datetime

import requests

from github_poster.loader.base_loader import BaseLoader, LoadError
from github_poster.loader.config import (
    NS_CLAENDAR_URL,
    NS_CLIENT_ID,
    NS_GRANT_TYPE,
    NS_TOKEN_URL,
)


class NSLoader(BaseLoader):
    unit = "mins"

    def __init__(self, from_year, to_year, _type, **kwargs):
        super().__init__(from_year, to_year, _type)
        self.session_token = kwargs.get("ns_session_token", "")
        self.device_id = kwargs.get("ns_device_id", "")
        self.headers = {
            "x-moon-os-language": "en-US",
            "x-moon-app-language": "en-US",
            "authorization": "",
            "x-moon-app-internal-version": "310",
            "x-moon-app-display-version": "1.16.0",
            "x-moon-os": "IOS",
            "accept-encoding": "gzip;q=1.0, compress;q=0.5",
            "accept-language": "en-US;q=1.0",
            "user-agent": "moon_ios/1.16.0 (com.nintendo.znma; build:310; iOS 15.1.0) "
            "Alamofire/1.16.0",
            "x-moon-timezone": "America/Los_Angeles",
        }
        self.s = requests.Session()

    @classmethod
    def add_loader_arguments(cls, parser, optional):
        parser.add_argument(
            "--ns_device_id",
            dest="ns_device_id",
            type=str,
            required=optional,
            help="",
        )
        parser.add_argument(
            "--ns_session_token",
            dest="ns_session_token",
            type=str,
            required=optional,
            help="",
        )

    def _make_access_headers(self):
        r = self.s.post(
            NS_TOKEN_URL,
            data={
                "session_token": self.session_token,
                "client_id": NS_CLIENT_ID,
                "grant_type": NS_GRANT_TYPE,
            },
        )
        if not r.ok:
            raise LoadError("can not get ns access token")
        access = r.json()
        self.headers["authorization"] = (
            access["token_type"] + " " + access["access_token"]
        )

    def get_api_data(self):
        self._make_access_headers()
        month_list = self.make_month_list()
        for m in month_list:
            # no data for this month for ns
            if m.month == datetime.now().month:
                continue
            r = self.s.get(
                NS_CLAENDAR_URL.format(
                    device_id=self.device_id,
                    # ns month format
                    month=m.to_date_string()[:7],
                ),
                headers=self.headers,
            )
            if not r.ok:
                print(f"Get ns calendar api failed {str(r.text)}")
            try:
                yield from list(r.json()["dailySummaries"].values())
            except Exception:
                # just pass for now
                pass

    def make_track_dict(self):
        data_list = self.get_api_data()
        for d in data_list:
            number = d.get("playingTime", 0)
            if number:
                minutes = int(number / 60)
                self.number_by_date_dict[d["date"]] = minutes
                self.number_list.append(minutes)

    def get_all_track_data(self):
        self.make_track_dict()
        self.make_special_number()
        return self.number_by_date_dict, self.year_list



class ValueRange:
    def __init__(self):
        self._lower = None
        self._upper = None

    @classmethod
    def from_pair(cls, value1, value2):
        r = cls()
        r.extend(value1)
        r.extend(value2)
        return r

    def is_valid(self):
        return self._lower is not None

    def lower(self):
        return self._lower

    def upper(self):
        return self._upper

    def diameter(self):
        if self.is_valid():
            return self.upper() - self.lower()
        return 0

    def contains(self, value):
        return self.is_valid() and (self.lower() <= value <= self.upper())

    def extend(self, value):
        if not self.is_valid():
            self._lower = value
            self._upper = value
        else:
            self._lower = min(self._lower, value)
            self._upper = max(self._upper, value)


class XY:
    """
    Represent x,y coords with properly overloaded operations.
    """

    def __init__(self, x=0, y=0):
        self.x = x
        self.y = y

    def __mul__(self, factor):
        if isinstance(factor, XY):
            return XY(self.x * factor.x, self.y * factor.y)
        return XY(self.x * factor, self.y * factor)

    def __rmul__(self, factor):
        if isinstance(factor, XY):
            return XY(self.x * factor.x, self.y * factor.y)
        return XY(self.x * factor, self.y * factor)

    def __add__(self, other):
        return XY(self.x + other.x, self.y + other.y)

    def __sub__(self, other):
        return XY(self.x - other.x, self.y - other.y)

    def __repr__(self):
        return f"XY: {self.x}/{self.y}"

    def tuple(self):
        return self.x, self.y



import re
from itertools import count as itercount
from itertools import takewhile

import colour


def interpolate_color(color1, color2, ratio):
    if ratio < 0:
        ratio = 0
    elif ratio > 1:
        ratio = 1
    c1 = colour.Color(color1)
    c2 = colour.Color(color2)
    c3 = colour.Color(
        hue=((1 - ratio) * c1.hue + ratio * c2.hue),
        saturation=((1 - ratio) * c1.saturation + ratio * c2.saturation),
        luminance=((1 - ratio) * c1.luminance + ratio * c2.luminance),
    )
    return c3.hex_l


def parse_years(s):
    """Parse a plaintext range of years into a pair of years

    Attempt to turn the input string into a pair of year values, from_year and to_year.
    If one year is passed, both from_year and to_year will be set to that year.
    If a range like '2016-2018' is passed, from_year will be set to 2016,
    and to_year will be set to 2018.

    Args:
        s: A string representing a range of years or a single year
    """
    m = re.match(r"^\d+$", s)
    if m:
        from_year = int(s)
        to_year = from_year
        return from_year, to_year
    m = re.match(r"^(\d+)-(\d+)$", s)
    if m:
        y1, y2 = int(m.group(1)), int(m.group(2))
        if y1 <= y2:
            from_year = y1
            to_year = y2
        else:
            from_year = y2
            to_year = y1
    return from_year, to_year


def make_key_times(year_count):
    """
    year_count: year run date count
    return: list of key times points
    should append `1` because the svg keyTimes rule
    """
    s = list(takewhile(lambda n: n < 1, itercount(0, 1 / year_count)))
    s.append(1)
    return [str(round(i, 2)) for i in s]



import twint

from github_poster.loader.base_loader import BaseLoader


class TwitterLoader(BaseLoader):
    track_color = "#1C9CEA"
    unit = "tweets"

    def __init__(self, from_year, to_year, _type, **kwargs):
        super().__init__(from_year, to_year, _type)
        self.user_name = kwargs.get("twitter_user_name", "")
        self.c = twint.Config()

    @classmethod
    def add_loader_arguments(cls, parser, optional):
        parser.add_argument(
            "--twitter_user_name",
            dest="twitter_user_name",
            type=str,
            required=optional,
            help="",
        )

    def get_api_data(self):
        self.c.Username = self.user_name
        self.c.Custom["tweet"] = ["id"]
        self.c.Custom["user"] = ["bio"]
        self.c.Store_object = True
        self.c.Since = f"{self.from_year}-01-01"
        self.c.Until = f"{self.to_year}-12-31"
        twint.run.Search(self.c)
        return twint.output.tweets_list

    def make_track_dict(self):
        data_list = self.get_api_data()
        for d in data_list:
            date_str = d.datetime[:10]
            self.number_by_date_dict[date_str] += 1
        for _, v in self.number_by_date_dict.items():
            self.number_list.append(v)

    def get_all_track_data(self):
        self.make_track_dict()
        self.make_special_number()
        return self.number_by_date_dict, self.year_list



from http.cookies import SimpleCookie

import requests

from github_poster.html_parser import parse_kindle_text_to_list
from github_poster.loader.base_loader import BaseLoader, LoadError
from github_poster.loader.config import (
    KINDLE_CN_HISTORY_URL,
    KINDLE_HEADER,
    KINDLE_HISTORY_URL,
)


class KindleLoader(BaseLoader):
    track_color = "#2A4A7B"
    unit = "days"

    def __init__(self, from_year, to_year, _type, **kwargs):
        super().__init__(from_year, to_year, _type)
        self.kindle_cookie = kwargs.get("kindle_cookie", "")
        self.session = requests.Session()
        self.header = KINDLE_HEADER
        self.is_cn = kwargs.get("cn", False)
        self.KINDLE_URL = KINDLE_CN_HISTORY_URL if self.is_cn else KINDLE_HISTORY_URL
        self._make_years_list()

    @classmethod
    def add_loader_arguments(cls, parser, optional):
        parser.add_argument(
            "--kindle_cookie",
            dest="kindle_cookie",
            type=str,
            required=optional,
            help="",
        )

    def _parse_kindle_cookie(self):
        cookie = SimpleCookie()
        cookie.load(self.kindle_cookie)
        cookies_dict = {}
        cookiejar = None
        for key, morsel in cookie.items():
            cookies_dict[key] = morsel.value
            cookiejar = requests.utils.cookiejar_from_dict(
                cookies_dict, cookiejar=None, overwrite=True
            )
        return cookiejar

    def get_api_data(self):
        r = self.session.get(self.KINDLE_URL, headers=self.header)
        if not r.ok:
            raise LoadError("Can not get kindle calendar data, please check cookie")
        yield from parse_kindle_text_to_list(r.text)

    def make_track_dict(self):
        data_list = self.get_api_data()
        for d in data_list:
            self.number_by_date_dict[d] = 1
            self.number_list.append(1)

    def get_all_track_data(self):
        self.session.cookies = self._parse_kindle_cookie()
        self.make_track_dict()
        self.make_special_number()
        return self.number_by_date_dict, self.year_list



from collections import defaultdict

from garminconnect import Garmin

from github_poster.loader.base_loader import BaseLoader


class GarminLoader(BaseLoader):
    track_color = "#E17A05"
    unit = "km"

    def __init__(self, from_year, to_year, _type, **kwargs):
        super().__init__(from_year, to_year, _type)
        self.before = None
        self.after = None
        self.number_by_date_dict = defaultdict(float)
        self.garmin_user_name = kwargs.get("garmin_user_name", "")
        self.garmin_password = kwargs.get("garmin_password", "")
        self.is_cn = kwargs.get("cn", False)
        self.client = None

    @classmethod
    def add_loader_arguments(cls, parser, optional):
        parser.add_argument(
            "--garmin_user_name",
            dest="garmin_user_name",
            type=str,
            required=optional,
            help="The username of Garmin",
        )
        parser.add_argument(
            "--garmin_password",
            dest="garmin_password",
            type=str,
            required=optional,
            help="The password of Garmin",
        )

    def _get_access(self):
        try:
            self.client = Garmin(
                self.garmin_user_name, self.garmin_password, is_cn=self.is_cn
            )
        except Exception as e:  # try twice
            print(str(e))
            print("try login again")
            self.client = Garmin(
                self.garmin_user_name, self.garmin_password, is_cn=self.is_cn
            )

    def get_api_data(self, start=0, limit=100):
        if self.client is None:
            self._get_access()
        activities = self.client.get_activities(start=start, limit=limit)
        if activities:
            # maybe some other way to do this
            yield from activities
            yield from self.get_api_data(start=start + limit, limit=limit)

    def make_track_dict(self):
        for activity in self.get_api_data():
            date = activity.get("startTimeLocal", None)
            distance = activity.get("distance", 0)
            if date and distance:
                date = date[:10]
                distance = round(distance, 2)
                self.number_by_date_dict[date] += distance / 1000

    def get_all_track_data(self):
        self.make_track_dict()
        self.make_special_number()
        return self.number_by_date_dict, self.year_list



from github_poster.utils import interpolate_color, make_key_times, parse_years


def test_interpolate_color():
    assert interpolate_color("#000000", "#ffffff", 0) == "#000000"
    assert interpolate_color("#000000", "#ffffff", 1) == "#ffffff"
    assert interpolate_color("#000000", "#ffffff", 0.5) == "#7f7f7f"
    assert interpolate_color("#000000", "#ffffff", -100) == "#000000"
    assert interpolate_color("#000000", "#ffffff", 12345) == "#ffffff"


def test_parse_years():
    assert parse_years("2012") == (2012, 2012)
    assert parse_years("2015-2021") == (2015, 2021)
    assert parse_years("2021-2015") == (2015, 2021)


def test_make_key_times():
    assert make_key_times(5) == ["0", "0.2", "0.4", "0.6", "0.8", "1"]



import json

import pendulum
import requests

from github_poster.loader.base_loader import BaseLoader, LoadError
from github_poster.loader.config import FOREST_CLAENDAR_URL, FOREST_LOGIN_URL


class ForestLoader(BaseLoader):
    unit = "trees"

    def __init__(self, from_year, to_year, _type, **kwargs):
        super().__init__(from_year, to_year, _type)
        self.user_email = kwargs.get("forest_email", "")
        self.password = kwargs.get("forest_password", "")
        self.user_id = None
        self.s = requests.Session()

    @classmethod
    def add_loader_arguments(cls, parser, optional):
        parser.add_argument(
            "--forest_email",
            dest="forest_email",
            type=str,
            required=optional,
            help="The email of forest",
        )
        parser.add_argument(
            "--forest_password",
            dest="forest_password",
            type=str,
            required=optional,
            help="The password of Forest",
        )

    def login(self):
        data = {"session": {"email": self.user_email, "password": self.password}}
        headers = {"Content-Type": "application/json"}
        r = self.s.post(FOREST_LOGIN_URL, headers=headers, data=json.dumps(data))
        if not r.ok:
            raise LoadError(f"Someting is wrong to login -- {r.text}")
        self.user_id = r.json()["user_id"]

    def get_api_data(self):
        date = str(self.from_year) + "-01-01"
        r = self.s.get(FOREST_CLAENDAR_URL.format(date=date, user_id=self.user_id))
        if not r.ok:
            raise LoadError(f"Someting is wrong to get data-- {r.text}")
        return r.json()["plants"]

    def make_track_dict(self):
        data_list = self.get_api_data()
        for d in data_list:
            date = pendulum.parse(d["created_at"], tz=self.time_zone).to_date_string()
            self.number_by_date_dict[date] += d["tree_count"]
        for _, v in self.number_by_date_dict.items():
            self.number_list.append(v)

    def get_all_track_data(self):
        self.login()
        self.make_track_dict()
        self.make_special_number()
        return self.number_by_date_dict, self.year_list



from abc import ABC, abstractmethod
from collections import defaultdict
from datetime import datetime

import pendulum
import pytz

from github_poster.loader.config import ARGUMENT_DICT, TIME_ZONE


class LoadError(Exception):
    pass


class BaseLoader(ABC):

    parser_loader_list = []
    #: The track color of the poster
    track_color = None
    #: The unit used by the poster
    unit = "times"

    def __init__(self, from_year, to_year, _type, **kwargs):
        assert to_year >= from_year
        self.from_year = from_year
        self.to_year = to_year
        self._type = _type
        self.time_zone = TIME_ZONE
        self.number_by_date_dict = defaultdict(int)
        self.special_number1 = None
        self.special_number2 = None
        self.number_list = []
        self.year_list = self._make_years_list()

    def _make_years_list(self):
        return list(range(int(self.from_year), int(self.to_year) + 1))

    def make_month_list(self):
        start = pendulum.datetime(self.from_year, 1, 1)
        end = pendulum.datetime(self.to_year, 12, 31)
        period = pendulum.period(start, end)
        month_list = list(period.range("months"))
        # filter
        month_list = [m for m in month_list if m < pendulum.now()]
        return month_list

    def make_special_number(self):
        """
        This func is to make special color number for poster
        special_number1 top 20%
        special_number2  top 20 % - 50%
        """
        # before python below 3.5 maybe need to sort
        number_list_set = sorted(list(set(self.number_list)))
        number_list_set_len = len(number_list_set)
        if number_list_set_len < 3:
            self.special_number1 = self.special_number2 = float("inf")
            return
        elif len(self.number_list) < 10:
            self.special_number1 = number_list_set[-1]
            self.special_number2 = number_list_set[-2]
        else:
            self.special_number1 = number_list_set[-1 * int(number_list_set_len * 0.2)]
            self.special_number2 = number_list_set[-1 * int(number_list_set_len * 0.50)]

    def adjust_time(self, time):
        tc_offset = datetime.now(pytz.timezone(self.time_zone)).utcoffset()
        return time + tc_offset

    @classmethod
    def add_arguments(cls, parser, optional):
        loader_group = parser.add_argument_group("Loader Arguments")
        cls.add_loader_arguments(loader_group, optional)
        group = parser.add_argument_group("Common Arguments")
        group.add_argument(
            "--year",
            metavar="YEAR",
            type=str,
            default=str(datetime.now().year),
            help='Filter tracks by year; "NUM", "NUM-NUM", "all" (default: all years)',
        )
        group.add_argument(
            "--me",
            metavar="NAME",
            type=str,
            default="Joey",
            help='User name to display (default: "Joey").',
        )
        group.add_argument(
            "--background-color",
            dest="background_color",
            metavar="COLOR",
            type=str,
            default="#222222",
            help='Background color of poster (default: "#222222").',
        )
        group.add_argument(
            "--track-color",
            dest="track_color",
            metavar="COLOR",
            type=str,
            default="#4DD2FF",
            help='Color of tracks (default: "#4DD2FF").',
        )
        group.add_argument(
            "--text-color",
            dest="text_color",
            metavar="COLOR",
            type=str,
            default="#FFFFFF",
            help='Color of text (default: "#FFFFFF").',
        )
        group.add_argument(
            "--special-color1",
            dest="special_color1",
            metavar="COLOR",
            default="yellow",
            help='Special track color (default: "yellow").',
        )
        group.add_argument(
            "--special-color2",
            dest="special_color2",
            metavar="COLOR",
            default="red",
            help="Secondary color of special tracks (default: red).",
        )
        group.add_argument(
            "--special-number1",
            dest="special_number1",
            type=float,
            default=0,
            help="Special number 1",
        )
        group.add_argument(
            "--special-number2",
            dest="special_number2",
            type=float,
            default=0,
            help="Special number 2",
        )
        group.add_argument(
            "--with-animation",
            dest="with_animation",
            action="store_true",
            help="add animation to the poster",
        )
        group.add_argument(
            "--animation-time",
            dest="animation_time",
            type=int,
            default=10,
            help="animation duration (default: 10s)",
        )
        # skyline args here
        group.add_argument(
            "--with-skyline",
            dest="with_skyline",
            action="store_true",
            help="with skyline(stl file)",
        )
        group.add_argument(
            "--skyline-with-name",
            dest="skyline_with_name",
            action="store_true",
            help="print name on skyline",
        )
        group.add_argument(
            "--skyline-year",
            dest="skyline_year",
            type=str,
            default="",
            help="the year to generate skyline",
        )
        # circular args here
        group.add_argument(
            "--is-circular",
            dest="is_circular",
            action="store_true",
            help="if is circular poster",
        )
        # title ignore type name
        group.add_argument(
            "--without-type-name",
            dest="without_type_name",
            default=True,
            action="store_false",
            help="if poster title with type name",
        )

        # is_cn here
        group.add_argument(
            "--cn",
            dest="cn",
            action="store_true",
            help="if accout is CN",
        )

    @classmethod
    def add_loader_arguments(cls, parser, optional):
        pass

    @abstractmethod
    def make_track_dict(self):
        pass

    @abstractmethod
    def get_all_track_data(self):
        pass



import json
import time
from http.cookies import SimpleCookie
from random import randint
from re import compile

import pendulum
import requests

from github_poster.html_parser.jike_parse import (
    find_count_dict_by_type_in_html,
    find_date_in_response,
    find_last_id_in_html,
    find_last_id_in_response,
)
from github_poster.loader.base_loader import BaseLoader, LoadError
from github_poster.loader.config import JIKE_GRAPHQL_URL, JIKE_PERSON_URL


class JikeLoader(BaseLoader):
    track_color = "#FFE411"
    unit = "times"

    def __init__(self, from_year, to_year, _type, **kwargs):
        super().__init__(from_year, to_year, _type)
        self.user_id = kwargs.get("user_id", "")
        self.jike_cookie = kwargs.get("jike_cookie", "")
        self.count_type = kwargs.get("count_type", "")
        self.session = requests.Session()
        self.headers = {
            "authority": "web-api.okjike.com",
            "accept": "*/*",
            "dnt": "1",
            "content-type": "application/json",
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": '"Windows"',
            "origin": "https://web.okjike.com",
            "sec-fetch-site": "same-site",
            "sec-fetch-mode": "cors",
            "sec-fetch-dest": "empty",
            "accept-language": "zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7",
        }
        # not more then 3 times for some day in svg are last year cells
        self.stop_time = 0

    @classmethod
    def add_loader_arguments(cls, parser, optional):
        parser.add_argument(
            "--jike_user_id",
            dest="user_id",
            type=str,
            help="The user id of jike",
        )
        parser.add_argument(
            "--jike_cookie",
            dest="jike_cookie",
            type=str,
            required=optional,
            help="The cookie for the jike website(XHR)",
        )
        parser.add_argument(
            "--count_type",
            dest="count_type",
            type=str,
            default="record",
            choices=["record", "like", "comment", "repost", "share"],
            help="""
            The count type of jike post,
            such as 'like' or 'comment' or 'repost' or 'share'
            """,
        )

    def _parse_jike_cookie(self):
        cookie = SimpleCookie()
        cookie.load(self.jike_cookie)
        cookies_dict = {}
        cookiejar = None
        for key, morsel in cookie.items():
            cookies_dict[key] = morsel.value
            cookiejar = requests.utils.cookiejar_from_dict(
                cookies_dict, cookiejar=None, overwrite=True
            )
        return cookiejar

    def _get_first_last_id(self):
        """
        get first last id for first post request
        """
        r = self.session.get(
            JIKE_PERSON_URL.format(user_id=self.user_id), headers=self.headers
        )
        if not r.ok:
            raise LoadError("Can not get first last id, please check your cookie")
        else:
            return find_last_id_in_html(r.text), find_count_dict_by_type_in_html(
                r.text, self.count_type
            )

    def _request_post_data(self, last_id="", resp_data_list=None):
        """
        request next page posts
        """
        if resp_data_list is None:
            resp_data_list = []

        payload_data = {
            "operationName": "UserFeeds",
            "variables": {"username": "", "loadMoreKey": {"lastId": ""}},
            "query": """
                query UserFeeds($username: String!, $loadMoreKey: JSON) {
                userProfile(username: $username) {
                    username
                    feeds(loadMoreKey: $loadMoreKey) {
                    ...BasicFeedItem
                    __typename
                    }
                    __typename
                }
                }

                fragment BasicFeedItem on FeedsConnection {
                pageInfo {
                    loadMoreKey
                    hasNextPage
                    __typename
                }
                nodes {
                    ... on MessageEssential {
                    ...FeedMessageFragment
                    __typename
                    }
                    __typename
                }
                __typename
                }

                fragment FeedMessageFragment on MessageEssential {
                ...EssentialFragment
                ... on OriginalPost {
                    ...LikeableFragment
                    ...CommentableFragment
                    __typename
                }
                }

                fragment EssentialFragment on MessageEssential {
                type
                shareCount
                repostCount
                createdAt
                }

                fragment LikeableFragment on LikeableMessage {
                liked
                likeCount
                __typename
                }

                fragment CommentableFragment on CommentableMessage {
                commentCount
                __typename
                }
            """,
        }
        # set lastId
        payload_data["variables"]["loadMoreKey"]["lastId"] = last_id
        payload_data["variables"]["username"] = self.user_id

        r = self.session.post(
            JIKE_GRAPHQL_URL, headers=self.headers, data=json.dumps(payload_data)
        )
        resp_dates = find_date_in_response(r)
        next_last_id = find_last_id_in_response(r.text)

        resp_data_list.extend(r.json()["data"]["userProfile"]["feeds"]["nodes"])
        if self._check_if_stop(resp_dates) or not next_last_id:
            return resp_data_list
        else:
            time.sleep(randint(1, 3))
            return self._request_post_data(next_last_id, resp_data_list)

    def _check_if_stop(self, dates):
        """
        stop when content other year data
        """
        r_date = compile(str(self.from_year) + ".*?")
        filter_list = list(filter(r_date.match, dates))
        if len(filter_list) != len(dates):
            self.stop_time += 1
        return self.stop_time == 3

    def get_api_data(self):
        data_cache = []
        # get first last id and first data
        first_last_id, first_data_dict = self._get_first_last_id()
        if not first_last_id:
            raise LoadError("Can not get first last id, please check your cookie")
        if self._check_if_stop(first_data_dict.keys()):
            return data_cache
        else:
            # do post get more
            post_resp_dates = self._request_post_data(first_last_id)
            data_cache.extend(post_resp_dates)
            return data_cache, first_data_dict

    def make_track_dict(self):
        data_list, first_data_dict = self.get_api_data()
        # add first page data
        self.number_by_date_dict.update(first_data_dict)
        # add rest page data
        count_key = self.count_type + "Count"
        for data in data_list:
            post_date = data["createdAt"]
            date_str = pendulum.parse(post_date, tz=self.time_zone).to_date_string()
            if self.count_type == "record":
                self.number_by_date_dict[date_str] += 1
            else:
                self.number_by_date_dict[date_str] += (
                    data[count_key] if data.get(count_key) else 0
                )
        for _, v in self.number_by_date_dict.items():
            self.number_list.append(v)

    def get_all_track_data(self):
        self.session.cookies = self._parse_jike_cookie()
        self.make_track_dict()
        self.make_special_number()
        print("Thanks for being addicted to jike.")
        return self.number_by_date_dict, self.year_list



from .github_parser import GitHubParser
from .gitlab_parser import GitLabParser
from .kindle_parser import parse_kindle_text_to_list

__all__ = ("GitHubParser", "GitLabParser", "parse_kindle_text_to_list")



import json
from re import findall


def parse_kindle_text_to_list(text):
    """
    whole html file js -> [date1, date2, date3.........]
    """
    r = findall('"days_read":(.*),"goal_info"', text)
    if r:
        return json.loads(r[0])
    return []



import json
from collections import defaultdict
from re import compile, findall

import pendulum

from github_poster.loader.config import TIME_ZONE


def find_last_id_in_response(text):
    """
    get last id from response
    """
    json_data = json.loads(text)
    load_more_key = json_data["data"]["userProfile"]["feeds"]["pageInfo"]["loadMoreKey"]
    return "" if load_more_key is None else load_more_key["lastId"]


def find_last_id_in_html(text):
    """
    find last id in html
    "json":{"lastId":"xxx"}}
    """
    r = findall('"json":{"lastId":"(.*?)"}', text)
    return r[0] if r else ""


def find_dateTime_in_html(text):
    """
    find dateTime in html
    """
    r = findall('<time dateTime="(.*?)">', text)
    if r:
        return r
    return []


def find_count_dict_by_type_in_html(text, count_type):
    """
    find count by type in html:
    likeCount, commentCount, shareCount, repostCount, recordCount
    """
    number_by_date_dict = defaultdict(int)
    # find all post data
    posts = findall('"__typename".*?"readTrackInfo"', text)
    if posts:
        r_count = compile(count_type + "Count.*?(\\d+)")
        create_at = compile('"createdAt":"(.*?)"')
        for post in posts:
            # find count by type
            date = create_at.findall(post)[0]
            if date:
                date_str = pendulum.parse(date, tz=TIME_ZONE).to_date_string()
                if count_type == "record":
                    count = 1
                else:
                    count = r_count.findall(post)[0]
                number_by_date_dict[date_str] += int(count)
    return number_by_date_dict


def find_date_in_response(r):
    date_list = []
    for item in r.json()["data"]["userProfile"]["feeds"]["nodes"]:
        date_list.append(item["createdAt"])
    return date_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from datetime import datetime

import requests

from github_poster.loader.base_loader import BaseLoader
from github_poster.loader.config import DOTA2_CALENDAR_API


class Dota2Loader(BaseLoader):
    track_color = "#567433"
    unit = "games"

    def __init__(self, from_year, to_year, _type, **kwargs):
        super().__init__(from_year, to_year, _type)
        self.dota2_id = kwargs.get("dota2_id", "")

    @classmethod
    def add_loader_arguments(cls, parser, optional):
        parser.add_argument(
            "--dota2_id",
            dest="dota2_id",
            type=str,
            required=optional,
            help="Check your dota2-id in-game or on the website(steamid32): "
            "https://steamid.xyz/",
        )

    def get_api_data(self):
        r = requests.get(
            DOTA2_CALENDAR_API.format(
                dota2_id=self.dota2_id,
            )
        )
        if not r.ok:
            print(f"get data2 calendar api failed {str(r.text)}")
            return []
        return r.json()

    def make_track_dict(self):
        data_list = self.get_api_data()
        for d in data_list:
            date = datetime.utcfromtimestamp(d["start_time"]).strftime("%Y-%m-%d")
            self.number_by_date_dict[date] += 1
        for _, v in self.number_by_date_dict.items():
            self.number_list.append(v)

    def get_all_track_data(self):
        self.make_track_dict()
        self.make_special_number()
        return self.number_by_date_dict, self.year_list



import time

import requests

from github_poster.loader.base_loader import BaseLoader
from github_poster.loader.config import WAKATIME_SUMMARY_URL


class WakaTimeLoader(BaseLoader):
    track_color = "#9BE9A8"
    unit = "mins"

    def __init__(self, from_year, to_year, _type, **kwargs):
        super().__init__(from_year, to_year, _type)
        self.wakatime_key = kwargs.get("wakatime_key", "")

    @classmethod
    def add_loader_arguments(cls, parser, optional):
        parser.add_argument(
            "--wakatime_key",
            dest="wakatime_key",
            type=str,
            required=optional,
            help="your wakatime api key here, "
            "more info: https://wakatime.com/settings/api-key",
        )

    def get_api_data(self):
        for year in range(self.from_year, self.to_year + 1):
            r = requests.get(
                WAKATIME_SUMMARY_URL.format(
                    wakatime_key=self.wakatime_key,
                    from_year="{}-01-01".format(year),
                    to_year="{}-12-31".format(year),
                )
            )
            if not r.ok:
                yield []
            data = r.json()
            yield from data["data"]
            # spider rule
            time.sleep(1)

    def make_track_dict(self):
        data_list = self.get_api_data()
        for d in data_list:
            if d:
                date = d["range"]["date"]
                self.number_by_date_dict[date] += int(
                    d["grand_total"]["total_seconds"] / 60.0
                )
        for _, v in self.number_by_date_dict.items():
            self.number_list.append(v)

    def get_all_track_data(self):
        self.make_track_dict()
        self.make_special_number()
        return self.number_by_date_dict, self.year_list



import requests

from github_poster.html_parser import GitHubParser
from github_poster.loader.base_loader import BaseLoader, LoadError
from github_poster.loader.config import GITHUB_CONTRIBUCTIONS_URL


class GitHubLoader(BaseLoader):
    track_color = "#9BE9A8"
    unit = "cons"

    def __init__(self, from_year, to_year, _type, **kwargs):
        super().__init__(from_year, to_year, _type)
        self.user_name = kwargs.get("github_user_name", "")

    @classmethod
    def add_loader_arguments(cls, parser, optional):
        parser.add_argument(
            "--github_user_name",
            dest="github_user_name",
            type=str,
            required=optional,
            help="",
        )

    def make_track_dict(self):
        for y in self.year_list:
            p = GitHubParser()
            try:
                r = requests.get(
                    GITHUB_CONTRIBUCTIONS_URL.format(
                        user_name=self.user_name,
                        start_day=f"{y}-01-01",
                        end_day=f"{y}-12-31",
                    )
                )
                self.number_by_date_dict.update(p.make_contribution_dict(r.text))
            except Exception as e:
                raise LoadError(f"Can not get GitHub contribuctions error: {str(e)}")
        for _, v in self.number_by_date_dict.items():
            self.number_list.append(v)

    def get_all_track_data(self):
        self.make_track_dict()
        self.make_special_number()
        return self.number_by_date_dict, self.year_list



from os import path

from setuptools import find_packages, setup

loc = path.abspath(path.dirname(__file__))

with open(loc + "/requirements.txt") as f:
    requirements = f.read().splitlines()

required = []
dependency_links = []

# Do not add to required lines pointing to Git repositories
EGG_MARK = "#egg="
for line in requirements:
    if (
        line.startswith("-e git:")
        or line.startswith("-e git+")
        or line.startswith("git:")
        or line.startswith("git+")
    ):
        line = line.lstrip("-e ")  # in case that is using "-e"
        if EGG_MARK in line:
            package_name = line[line.find(EGG_MARK) + len(EGG_MARK) :]
            repository = line[: line.find(EGG_MARK)]
            required.append("%s @ %s" % (package_name, repository))
            dependency_links.append(line)
        else:
            print("Dependency to a git repository should have the format:")
            print("git+ssh://git@github.com/xxxxx/xxxxxx#egg=package_name")
    else:
        required.append(line)

setup(
    name="github_poster",
    author="yihong0618",
    author_email="zouzou0208@gmail.com",
    url="https://github.com/yihong0618/GitHubPoster",
    license="MIT",
    version="1.9.2",
    description="Make everything a GitHub svg poster and Skyline!",
    packages=find_packages(),
    include_package_data=True,
    install_requires=required,
    dependency_links=dependency_links,
    entry_points={
        "console_scripts": ["github_poster = github_poster.cli:main"],
    },
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from collections import defaultdict

from github_poster.loader.base_loader import BaseLoader
from github_poster.loader.bilibili_loader import BilibiliLoader
from github_poster.loader.leetcode_loader import LeetcodeLoader


class SummaryLoader(BaseLoader):
    def __init__(self, from_year, to_year, _type, **kwargs):
        super().__init__(from_year, to_year, _type)
        self.types = kwargs.get("types", "")
        self.type_summary_dict = {}
        self.loader_list = []

    def set_loader_list(self, loader):
        self.loader_list.append(loader)

    @classmethod
    def add_loader_arguments(cls, parser, optional):

        parser.add_argument(
            "--types",
            dest="types",
            type=str,
            required=True,
            help="All types you want to generate summary, split by comma",
        )

        for l in cls.parser_loader_list:
            l.add_loader_arguments(parser, optional)

    def get_api_data(self):
        pass

    def make_track_dict(self):
        pass

    def get_all_track_data(self):
        pass



import html.parser


class GitLabParser(html.parser.HTMLParser):
    def __init__(self):
        html.parser.HTMLParser.__init__(self)
        self.lis = []

    def handle_starttag(self, tag, attrs):
        if tag == "li":
            self.lis.append(1)



import time

import pendulum
import requests

from github_poster.loader.base_loader import BaseLoader
from github_poster.loader.config import (
    LEETCODE_CN_SUBMISSIONS_URL,
    LEETCODE_SUBMISSIONS_URL,
)


class LeetcodeLoader(BaseLoader):
    unit = "subs"

    def __init__(self, from_year, to_year, _type, **kwargs):
        super().__init__(from_year, to_year, _type)
        self.leetcode_cookie = kwargs.get("leetcode_cookie", "")
        self.is_cn = kwargs.get("cn", False)
        self.LEETCODE_URL = (
            LEETCODE_CN_SUBMISSIONS_URL if self.is_cn else LEETCODE_SUBMISSIONS_URL
        )

    @classmethod
    def add_loader_arguments(cls, parser, optional):
        parser.add_argument(
            "--leetcode_cookie",
            dest="leetcode_cookie",
            type=str,
            required=optional,
            help="",
        )

    def get_api_data(self):
        offset = 0
        last_key = ""
        # leetcode may cause the permission error, just retry
        error_times = 0
        while 1:
            try:
                r = requests.get(
                    self.LEETCODE_URL.format(offset=offset, last_key=last_key),
                    cookies={"cookie": self.leetcode_cookie},
                )
                if not r.ok:
                    print(r.text)
                    break
                data = r.json()
                submissions = data["submissions_dump"]
                last_submission_this_api = submissions[-1]
                last_year = pendulum.from_timestamp(
                    last_submission_this_api["timestamp"]
                ).year
                if not submissions or last_year < self.from_year:
                    break
                yield from submissions
                if not data["has_next"]:
                    break
                last_key = data["last_key"]
                offset += 20
                time.sleep(2)
            except Exception:
                error_times += 1
                print(f"error times {error_times}")
                if error_times > 3:
                    break

    def make_track_dict(self):
        data_list = self.get_api_data()
        for d in data_list:
            if d:
                date = pendulum.from_timestamp(d["timestamp"]).to_date_string()
                self.number_by_date_dict[date] += 1
        for _, v in self.number_by_date_dict.items():
            self.number_list.append(v)

    def get_all_track_data(self):
        self.make_track_dict()
        self.make_special_number()
        return self.number_by_date_dict, self.year_list



import calendar
import datetime
import math

import svgwrite
from svgwrite import animate

from github_poster.drawer import Drawer
from github_poster.err import CircularDrawError
from github_poster.structures import XY, ValueRange
from github_poster.utils import make_key_times


class CircularDrawer(Drawer):

    # somecode from https://github.com/flopp/GpxTrackPoster
    name = "circular"

    def __init__(self, p):
        super().__init__(p)

    def draw(self, dr, size, is_summary=False):
        # TODO: make a summary circle
        if self.poster.tracks is None:
            raise CircularDrawError("No tracks to draw.")
        if self.poster.length_range_by_date is None:
            return
        margin = XY(10, 10)
        key_times = make_key_times(len(self.poster.years))
        animate_index = 1
        for year in range(self.poster.years[0], self.poster.years[-1] + 1):
            values = ["0"] * len(key_times)
            values[animate_index] = "1"
            values_str = ";".join(values)
            key_animate = self._make_key_anmiate(values_str, key_times)
            self._draw_year(dr, size, margin, year, key_animate)
            animate_index += 1

    def _draw_year(self, dr, size, offset, year, key_animate):
        min_size = min(size.x, size.y)
        outer_radius = 0.5 * min_size - 6
        radius_range = ValueRange.from_pair(outer_radius / 4, outer_radius)
        center = offset + 0.5 * size

        year_style = f"dominant-baseline: central; font-size:{min_size * 4.0 / 80.0}px; \
                       font-family:Arial;"
        month_style = f"font-size:{min_size * 3.0 / 80.0}px; font-family:Arial;"

        year_text = dr.text(
            f"{year}",
            insert=(center.x, center.y),
            fill=self.poster.colors["text"],
            text_anchor="middle",
            alignment_baseline="middle",
            style=year_style,
        )
        year_text.add(key_animate)
        dr.add(year_text)

        year_length = self.poster.total_sum_year_dict.get(year, 0)
        year_length_str = str(int(year_length)) + f" {self.poster.units}"

        year_stats = dr.text(
            year_length_str,
            insert=(115 - len(str(year_length_str)), 8),  # TODO: make this dynamic
            fill=self.poster.colors["text"],
            text_anchor="middle",
            alignment_baseline="middle",
            style=year_style,
        )
        year_stats.add(key_animate)
        dr.add(year_stats)

        # user stats
        user_stats = dr.text(
            f"{self.poster.title}",
            insert=(5 + len(self.poster.title), 8),  # ToDO: make this dynamic
            fill=self.poster.colors["text"],
            text_anchor="middle",
            alignment_baseline="middle",
            style=year_style,
        )
        user_stats.add(key_animate)
        dr.add(user_stats)
        df = 360.0 / (366 if calendar.isleap(year) else 365)
        day = 0
        date = datetime.date(year, 1, 1)
        while date.year == year:
            text_date = date.strftime("%Y-%m-%d")
            a1 = math.radians(day * df)
            a2 = math.radians((day + 1) * df)
            if date.day == 1:
                (_, last_day) = calendar.monthrange(date.year, date.month)
                a3 = math.radians((day + last_day - 1) * df)
                sin_a1, cos_a1 = math.sin(a1), math.cos(a1)
                sin_a3, cos_a3 = math.sin(a3), math.cos(a3)
                r1 = outer_radius + 1
                r2 = outer_radius + 6
                r3 = outer_radius + 2
                line = dr.line(
                    start=(center + r1 * XY(sin_a1, -cos_a1)).tuple(),
                    end=(center + r2 * XY(sin_a1, -cos_a1)).tuple(),
                    stroke=self.poster.colors["text"],
                    stroke_width=0.3,
                )
                line.add(key_animate)
                dr.add(line)
                path = dr.path(
                    d=("M", center.x + r3 * sin_a1, center.y - r3 * cos_a1),
                    fill="none",
                    stroke="none",
                )
                path.push(
                    f"a{r3},{r3} 0 0,1 {r3 * (sin_a3 - sin_a1)}, \
                    {r3 * (cos_a1 - cos_a3)}"
                )
                path.add(key_animate)
                dr.add(path)
                tpath = svgwrite.text.TextPath(
                    path, date.strftime("%B"), startOffset=(0.5 * r3 * (a3 - a1))
                )
                text = dr.text(
                    "",
                    fill=self.poster.colors["text"],
                    text_anchor="middle",
                    style=month_style,
                )
                text.add(tpath)
                text.add(key_animate)
                dr.add(text)

            if text_date in self.poster.tracks:
                self._draw_circle_segment(
                    dr,
                    self.poster.tracks[text_date],
                    a1,
                    a2,
                    radius_range,
                    center,
                    key_animate,
                )
            day += 1
            date += datetime.timedelta(1)

    def _make_key_anmiate(self, values, key_times):
        dur = len(key_times)
        return animate.Animate(
            "opacity",
            dur=f"{dur}s",
            values=values,
            keyTimes=";".join(key_times),
            repeatCount="indefinite",
        )

    def _draw_circle_segment(
        self,
        dr,
        length,
        a1,
        a2,
        rr,
        center,
        key_animate,
    ):
        color = self.make_color(self.poster.length_range_by_date, length)
        color = self.make_color(self.poster.length_range_by_date, length)
        r1 = rr.lower()
        r2 = (
            rr.lower()
            + rr.diameter() * length / self.poster.length_range_by_date.upper()
        )
        sin_a1, cos_a1 = math.sin(a1), math.cos(a1)
        sin_a2, cos_a2 = math.sin(a2), math.cos(a2)
        path = dr.path(
            d=("M", center.x + r1 * sin_a1, center.y - r1 * cos_a1),
            fill=color,
            stroke="none",
        )
        path.push("l", (r2 - r1) * sin_a1, (r1 - r2) * cos_a1)
        path.push(f"a{r2},{r2} 0 0,0 {r2 * (sin_a2 - sin_a1)},{r2 * (cos_a1 - cos_a2)}")
        path.push("l", (r1 - r2) * sin_a2, (r2 - r1) * cos_a2)
        path.add(key_animate)
        dr.add(path)



import datetime
import json
import time
from collections import defaultdict

import requests

from github_poster.loader.base_loader import BaseLoader
from github_poster.loader.config import (
    NIKE_BASE_URL,
    NIKE_CLIENT_ID,
    NIKE_TOKEN_REFRESH_URL,
)


class NRCLoader(BaseLoader):
    track_color = "#F4ED5E"
    unit = "km"

    def __init__(self, from_year, to_year, _type, **kwargs):
        super().__init__(from_year, to_year, _type)
        self.session = requests.Session()
        self.before = None
        self.after = None
        self.number_by_date_dict = defaultdict(float)
        self.nike_access = False
        self.nike_refresh_token = kwargs.get("nike_refresh_token", "")

    @classmethod
    def add_loader_arguments(cls, parser, optional):
        parser.add_argument(
            "--nike_refresh_token",
            dest="nike_refresh_token",
            type=str,
            required=optional,
            help="",
        )

    def _get_access(self):
        try:
            r = self.session.post(
                NIKE_TOKEN_REFRESH_URL,
                data=json.dumps(
                    {
                        "refresh_token": self.nike_refresh_token,
                        "client_id": NIKE_CLIENT_ID,
                        "grant_type": "refresh_token",
                    }
                ),
            )
        except Exception as e:
            print(str(e))
            raise Exception("Nike Auth failed please check your refresh token")
        access_token = r.json()["access_token"]
        self.session.headers.update({"Authorization": f"Bearer {access_token}"})

    def get_api_data(self, timestamp=0):
        if not self.nike_access:
            self._get_access()

        last_id = None
        while True:
            if last_id is not None:
                url = f"{NIKE_BASE_URL}/activities/after_id/{last_id}"
            else:
                url = f"{NIKE_BASE_URL}/activities/after_time/{timestamp}"
            r = self.session.get(url)
            if not r.ok:
                raise Exception("Get activities failed")
            data = r.json()
            last_id = data["paging"].get("after_id")
            activities = data["activities"]
            print(f"get NRC data since id {last_id}")

            for activity in activities:
                # ignore NTC record
                app_id = activity["app_id"]
                if app_id == "com.nike.ntc.brand.ios":
                    continue
                yield activity
            if last_id is None or not activities:
                return

    def make_track_dict(self):
        try:
            timestamp = int(
                time.mktime(
                    datetime.datetime.strptime(
                        f"01/01/{self.from_year}", "%d/%m/%Y"
                    ).timetuple()
                )
                * 1000
            )
        except Exception as e:
            print(str(e))
            timestamp = 0
        tracks = list(self.get_api_data(timestamp))
        for t in tracks:
            start_time = t["start_epoch_ms"]
            start_date = str(datetime.datetime.fromtimestamp(start_time / 1000).date())
            summaries = t["summaries"]
            distance = 0.0
            for summary in summaries:
                if summary.get("metric", "") == "distance":
                    distance = summary["value"]
                    distance = round(distance, 2)
                    self.number_by_date_dict[start_date] += distance
                    break
            self.number_list.append(distance)
        return tracks

    def get_all_track_data(self):
        self.make_track_dict()
        self.make_special_number()
        return self.number_by_date_dict, self.year_list



DOM_BOX_TUPLE = (2.6, 2.6)
DOM_BOX_TUPLE_LIST_FOR_TWO = ((2.6, 1.3), (2.6, 1.3))
DOM_BOX_TUPLE_LIST_FOR_THREE = ((2.7, 0.9), (2.7, 0.9), (2.7, 0.9))

# color from -> to (like douban color)
COLOR_TUPLE = (("#CAE3F7", "#266EDA"), ("#C9E8AF", "#8FD670"), ("#FEE0D7", "#F27A40"))

DOM_BOX_DICT = {
    1: {"dom": (DOM_BOX_TUPLE,)},
    2: {"dom": DOM_BOX_TUPLE_LIST_FOR_TWO},
    3: {"dom": DOM_BOX_TUPLE_LIST_FOR_THREE},
}

DEFAULT_DOM_COLOR = "#444444"
MONTH_NAMES = [
    "Jan",
    "Feb",
    "Mar",
    "Apr",
    "May",
    "Jun",
    "Jul",
    "Aug",
    "Sep",
    "Oct",
    "Nov",
    "Dec",
]

TYPE_INFO_DICT = {
    "duolingo": "Duolingo",
    "github": "GitHub",
    "leetcode": "LeetCode",
    "twitter": "Twitter",
    "youtube": "YouTube",
    "gitlab": "GitLab",
    "strava": "Strava",
    "bilibili": "bilibili",
    "nike": "Nike",
    "strava": "Strava",
    "ns": "NS",
    "kindle": "Kindle",
    "garmin": "Garmin",
    "shanbay": "Shanbay",
    "forest": "Forest",
    "jike": "Jike",
}



import math
import os

import numpy as np
from pendulum import parse, period
from sdf import X, Y, box, ease, measure_text, rectangle, text, union

import github_poster.skyline
from github_poster.skyline.config import (
    BASE_HEIGHT,
    BASE_LENGTH,
    BASE_TOP_LENGTH,
    BASE_TOP_WIDTH,
    BASE_WIDTH,
    BOX_DIMENSION,
)


class Skyline:
    def __init__(self, file_name, year, skyline_type, number_by_date_dict, user_name):
        self.file_name = file_name
        self.year = year
        self.skyline_type = skyline_type
        self.number_by_date_dict = number_by_date_dict
        self.type_info_dict = None
        assert len(user_name) < 16
        self.user_name = user_name

    def _make_box(self, box_height):
        b = box((BOX_DIMENSION, BOX_DIMENSION, box_height))
        # change to the base location
        b = b.translate(
            (BOX_DIMENSION / 2, BOX_DIMENSION / 2, box_height / 2 + BASE_HEIGHT)
        )
        return b

    def __make_one_year_dates(self):
        dates = list(period(parse(f"{self.year}-01-01"), parse(f"{self.year}-12-31")))
        return [d.to_date_string() for d in dates]

    def _make_skyline_boxes(self):
        box_list = []
        dates = self.__make_one_year_dates()
        # for test
        week_number = 0
        for i, j in enumerate(dates):
            day_number = i % 7
            if day_number == 0:
                week_number += 1
            value = self.number_by_date_dict.get(j)
            if value:
                box = self._make_box(value)
                box = box.translate(
                    (
                        3.5 + 2.5 + (week_number - 1) * BOX_DIMENSION,
                        3.5 + 2.5 + day_number * BOX_DIMENSION,
                        0,
                    )
                )
                box_list.append(box)
        return box_list

    def _make_skyline_card(self, text_info, offset=0):
        # TODO change the magic numbers
        # support change font
        FONT = os.path.join(github_poster.skyline.__path__[0], "font", "arial.ttf")
        TEXT = text_info
        w, h = measure_text(FONT, TEXT)
        t = (
            text(FONT, TEXT, 5 * (w / h), 5)
            .extrude(h)
            .translate((10 + w + offset, BASE_HEIGHT / 2, h / 2))
        )
        face_angle = math.tan(3.5 / 10)
        t = t.orient(Y).rotate(-face_angle, X)
        return t

    def _make_skyline_base(self):
        base = rectangle((BASE_LENGTH, BASE_WIDTH)).extrude_to(
            rectangle((BASE_TOP_LENGTH, BASE_TOP_WIDTH)), BASE_HEIGHT, ease.linear
        )
        # change to the base location
        base = base.translate((BASE_TOP_LENGTH / 2, BASE_WIDTH / 2, BASE_HEIGHT / 2))
        return base

    def _normalize_data(self):
        """
        normalize the values from 1 to 20
        """
        if self.number_by_date_dict:
            values_list = list(self.number_by_date_dict.values())
            values_list = (values_list - np.min(values_list)) / (
                np.max(values_list) - np.min(values_list)
            )
            values_list = [int(i * 20) + 1 for i in values_list]
            # only support for Python 3.6+, dict is ordered
            self.number_by_date_dict = dict(
                zip(self.number_by_date_dict.keys(), values_list)
            )

    def make_skyline(self):
        self._normalize_data()
        base = self._make_skyline_base()
        boxes = self._make_skyline_boxes()
        boxes = union(*boxes)
        text_info = self.type_info_dict.get(self.skyline_type, self.skyline_type)
        skyline_info_card = self._make_skyline_card(text_info)
        skyline_year_card = self._make_skyline_card(str(self.year), offset=120)
        skyline = base | boxes | skyline_info_card.k(0.25) | skyline_year_card.k(0.25)
        if self.user_name:
            skyline_name_card = self._make_skyline_card(self.user_name, offset=30)
            skyline = skyline | skyline_name_card
        skyline.save(self.file_name)
